package parkomat.model;

import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Tariffa implements Comparable<Tariffa> {
	private Map<DayOfWeek,List<FasciaTariffaria>> mappaFasce;
	private String nomeTariffa;
	private double costoOrario;
	private int franchigia, sostaMinima;


	public Tariffa(String nomeTariffa, double costoOrario, int franchigia, int sostaMinima) {
		this.nomeTariffa=nomeTariffa;
		this.costoOrario=costoOrario;
		this.franchigia = franchigia;
		this.sostaMinima = sostaMinima;
		this.mappaFasce = new HashMap<>();
		for (DayOfWeek day : DayOfWeek.values()) mappaFasce.put(day, new ArrayList<>());
		
	}

	public double getCostoOrario() {
		return costoOrario;
	}

	public int getFranchigia() {
		return franchigia;
	}

	public int getDurataSostaMinima() {
		return sostaMinima;
	}
	
	@Override
	public String toString() {
		return "Tariffa [" + nomeTariffa + ", costo orario=" + costoOrario +", franchigia=" + franchigia + ", minimo=" + sostaMinima + 
				"\n mappaFasce=" + mappaFasce + "]\n";
	}


	public Tariffa(String nomeTariffa, double costoOrario, int franchigia, int sostaMinima, Map<DayOfWeek,List<FasciaTariffaria>> fasce) {
		this.nomeTariffa=nomeTariffa;
		this.costoOrario=costoOrario;
		this.franchigia = franchigia;
		this.sostaMinima = sostaMinima;
		this.mappaFasce = fasce;
	}

	public Map<DayOfWeek,List<FasciaTariffaria>> getFasce() {
		return mappaFasce;
	}

	public String getNomeTariffa() {
		return nomeTariffa;
	}

	public List<FasciaTariffaria> getFasce(DayOfWeek day) {
		return mappaFasce.get(day);
	}

	public void addFascia(FasciaTariffaria fascia, DayOfWeek day) {
		addFascia(fascia, mappaFasce.get(day));
	}

	private void addFascia(FasciaTariffaria fascia, List<FasciaTariffaria> fasce) {
		if ( 	fasce.isEmpty() || 
				fasce.get(fasce.size()-1).getOraFinoA().minusMinutes(1).isBefore(fascia.getOraDa())	||
				fasce.get(fasce.size()-1).getOraFinoA().minusMinutes(1).equals(fascia.getOraDa()))  {
			//System.out.println("attuale " + fasce); // PRINT
			//System.out.println("adding  " + fascia); // PRINT
				fasce.add(fascia);
			//System.out.println("attuale " + fasce); // PRINT
		}
		else {
			for (int i=0; i<fasce.size(); i++) {
				FasciaTariffaria f = fasce.get(i);
				if (fascia.getOraFinoA().minusMinutes(1).isBefore(f.getOraDa()) || 
					fascia.getOraFinoA().minusMinutes(1).equals(f.getOraDa()) 	) {
					//	System.out.println("attuale " + fasce); // PRINT
					//	System.out.println("adding  " + fascia); // PRINT
					fasce.add(i, fascia);
					//	System.out.println("result " + fasce); // PRINT
					break;
				}
			}
		}
	}

	public boolean isValid(){
		for (DayOfWeek day : DayOfWeek.values()) {
			if (!isValid(day)) return false;
		}
		return true;
	}
	
	public boolean isValid(DayOfWeek day){
		List<FasciaTariffaria> fasce = mappaFasce.get(day);
		if (fasce==null || fasce.isEmpty()) return false;
		LocalTime current = LocalTime.of(0,0); // si parte da mezzanotte...
		for(FasciaTariffaria f : fasce) {
			// System.out.println("current  = " + current);
			// System.out.println("checking = " + f.getOraDa());
			if (!current.equals(f.getOraDa())) return false; 
			else current=f.getOraFinoA();//.plusMinutes(1); // es. fascia finisce 7:59, current diventa 8:00
		}
		return current.equals(LocalTime.of(0,0)); // .. e alla fine si deve aver chiuso il giro
	}	
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((nomeTariffa == null) ? 0 : nomeTariffa.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Tariffa that = (Tariffa) obj;
		return nomeTariffa.equals(that.nomeTariffa);
	}

	@Override
	public int compareTo(Tariffa that) {
		return this.nomeTariffa.compareTo(that.nomeTariffa);
	}
	
	
}
