package parkomat.tests.model;

import static org.junit.Assert.*;

import java.time.DayOfWeek;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import parkomat.model.*;
import parkomat.model.PagamentoAttivo;

public class TicketIssuerTests {
	
	private List<FasciaTariffaria> fasceFeriali;
	private Map<DayOfWeek, List<FasciaTariffaria>> mappaFasce;
	private Tariffa tariffaH1, tariffaH1f;
	private Map<String, TariffaValida> mappaTariffeValide;
	
	@Before
	public void setUp() {
		fasceFeriali = List.of(
				// nomeTariffa, giorno, daOra, aOra, costoOrario, franchigia, sostaMinima, stagionalita)
				new FasciaTariffaria(LocalTime.of(0,0), LocalTime.of(8,00), PagamentoAttivo.GRATUITO),
				new FasciaTariffaria(LocalTime.of(8,0), LocalTime.of(20,00),PagamentoAttivo.A_PAGAMENTO),
				new FasciaTariffaria(LocalTime.of(20,0), LocalTime.of(0,0), PagamentoAttivo.GRATUITO)
				); 
		mappaFasce = Map.of(
				DayOfWeek.MONDAY, fasceFeriali,
				DayOfWeek.TUESDAY, fasceFeriali,
				DayOfWeek.WEDNESDAY, fasceFeriali,
				DayOfWeek.THURSDAY, fasceFeriali,
				DayOfWeek.FRIDAY, fasceFeriali,
				DayOfWeek.SATURDAY, fasceFeriali,
				DayOfWeek.SUNDAY, fasceFeriali				
				);
		tariffaH1 = new Tariffa("H1", 0.50, 0, 60, mappaFasce);
		tariffaH1f = new Tariffa("H1f", 0.50, 60, 60, mappaFasce);
		
		mappaTariffeValide = Map.of(
				  "H1", new TariffaValida(tariffaH1),
				  "H1f", new TariffaValida(tariffaH1f)
				 );
	}
	
	
	@Test
	public void testSostaEntroIlMinimo1130_1150() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1.getNomeTariffa(), LocalDateTime.of(2018,06,15, 11,30), LocalDateTime.of(2018,06,15, 11,50));
		assertEquals("0,50\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaSingoloGiorno730_1500() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 15,00));
		assertEquals("3,50\u00A0€", tk.getCostoAsString());
	}
	
	@Test
	public void testSostaSingoloGiorno730_2100() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 21,00));
		assertEquals("6,00\u00A0€", tk.getCostoAsString());
	}
	
	@Test
	public void testSostaDueGiorni730d1_12d2() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,16, 12,00));
		assertEquals("8,00\u00A0€", tk.getCostoAsString());
	}
	
	@Test
	public void testSostaSingoloGiornoConFranchigia1h_700_1500() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1f.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,00), LocalDateTime.of(2018,06,15, 15,00));
		assertEquals("3,50\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaSingoloGiornoConFranchigia1h_730_1500() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1f.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 15,00));
		assertEquals("3,25\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaSingoloGiornoConFranchigia1h730_2100() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1f.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 21,00));
		assertEquals("5,75\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaDueGiornConFranchigia1hi730d1_12d2() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaH1f.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,16, 12,00));
		assertEquals("7,75\u00A0€", tk.getCostoAsString());
	}
}
