package parkomat.tests.model;

import static org.junit.Assert.*;

import java.time.DayOfWeek;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import parkomat.model.*;
import parkomat.model.PagamentoAttivo;

/** 
 *  This class tests the case of full-day, all-week payment, that is, no free periods, not even atnight or Sundays
 * */
public class TicketIssuerTests2 {
	
	private List<FasciaTariffaria> fasciaUnica;
	private Map<DayOfWeek, List<FasciaTariffaria>> mappaFasce;
	private Tariffa tariffaUnica, tariffaUnicaConFranchigia;
	private Map<String, TariffaValida> mappaTariffeValide;
	
	@Before
	public void setUp() {
		fasciaUnica = List.of(
				// nomeTariffa, giorno, daOra, aOra, costoOrario, franchigia, sostaMinima, stagionalita)
				new FasciaTariffaria(LocalTime.of(0,0), LocalTime.of(0,00), PagamentoAttivo.A_PAGAMENTO)
				); 
		mappaFasce = Map.of(
				DayOfWeek.MONDAY, fasciaUnica,
				DayOfWeek.TUESDAY, fasciaUnica,
				DayOfWeek.WEDNESDAY, fasciaUnica,
				DayOfWeek.THURSDAY, fasciaUnica,
				DayOfWeek.FRIDAY, fasciaUnica,
				DayOfWeek.SATURDAY, fasciaUnica,
				DayOfWeek.SUNDAY, fasciaUnica				
				);
		tariffaUnica = new Tariffa("Unica", 1.00, 0, 60, mappaFasce);
		tariffaUnicaConFranchigia = new Tariffa("UnicaConFranchigia", 1.00, 60, 60, mappaFasce);
		
		mappaTariffeValide = Map.of(
				  "Unica", new TariffaValida(tariffaUnica),
				  "UnicaConFranchigia", new TariffaValida(tariffaUnicaConFranchigia)
				 );
	}
	
	
	@Test
	public void testSostaEntroIlMinimo1130_1150() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnica.getNomeTariffa(), LocalDateTime.of(2018,06,15, 11,30), LocalDateTime.of(2018,06,15, 11,50));
		assertEquals("1,00\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaSingoloGiorno730_1500() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnica.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 15,00));
		assertEquals("7,50\u00A0€", tk.getCostoAsString());
	}
	
	@Test
	public void testSostaSingoloGiorno730_2100() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnica.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 21,00));
		assertEquals("13,50\u00A0€", tk.getCostoAsString());
	}
	
	@Test
	public void testSostaDueGiorni730d1_12d2() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnica.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,16, 12,00));
		assertEquals("28,50\u00A0€", tk.getCostoAsString());
	}
	
	@Test
	public void testSostaSingoloGiornoConFranchigia1h_700_1500() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnicaConFranchigia.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,00), LocalDateTime.of(2018,06,15, 15,00));
		assertEquals("7,00\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaSingoloGiornoConFranchigia1h_730_1500() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnicaConFranchigia.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 15,00));
		assertEquals("6,50\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaSingoloGiornoConFranchigia1h730_2100() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnicaConFranchigia.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,15, 21,00));
		assertEquals("12,50\u00A0€", tk.getCostoAsString());
	}

	@Test
	public void testSostaDueGiornConFranchigia1hi730d1_12d2() {
		MyTicketIssuer emitter = new MyTicketIssuer(mappaTariffeValide);
		Ticket tk = emitter.emitTicket(tariffaUnicaConFranchigia.getNomeTariffa(), LocalDateTime.of(2018,06,15, 7,30), LocalDateTime.of(2018,06,16, 12,00));
		assertEquals("27,50\u00A0€", tk.getCostoAsString());
	}
}
