package parkomat.tests.persistence;

import java.io.IOException;
import java.io.Reader;
import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import parkomat.model.FasciaTariffaria;
import parkomat.model.PagamentoAttivo;
import parkomat.model.Tariffa;
import parkomat.persistence.BadFileFormatException;
import parkomat.persistence.TariffeReader;

public class TariffeReaderMock implements TariffeReader {

	private List<FasciaTariffaria> fasceFeriali;
	private Map<DayOfWeek, List<FasciaTariffaria>> mappaFasce;
	private Tariffa tariffaH1;
	
	public TariffeReaderMock() {
		fasceFeriali = List.of(
				new FasciaTariffaria(LocalTime.of(0,0), LocalTime.of(7,59), PagamentoAttivo.GRATUITO),
				new FasciaTariffaria(LocalTime.of(8,0), LocalTime.of(19,59), PagamentoAttivo.A_PAGAMENTO),
				new FasciaTariffaria(LocalTime.of(20,0), LocalTime.of(23,59), PagamentoAttivo.GRATUITO)
				); 
		mappaFasce = Map.of(
				DayOfWeek.MONDAY, fasceFeriali,
				DayOfWeek.TUESDAY, fasceFeriali,
				DayOfWeek.WEDNESDAY, fasceFeriali,
				DayOfWeek.THURSDAY, fasceFeriali,
				DayOfWeek.FRIDAY, fasceFeriali,
				DayOfWeek.SATURDAY, fasceFeriali,
				DayOfWeek.SUNDAY, fasceFeriali				
				);
		mappaFasce = new HashMap<>(mappaFasce);
		tariffaH1 = new Tariffa("H1", 0.50, 0, 60, mappaFasce);
	}
	
	@Override
	public Map<String, Tariffa> leggiTariffe(Reader reader) throws IOException, BadFileFormatException {		
		return Map.of(
		  "H1", tariffaH1
		 );
	}

}
